<?php

function forward_post_handler() {

    // Get the new forward address, if any
    $newAddress = getvar('newAddress');

    // See what address should be removed, if any
    $removeAddress = getvar('removeAddress');
    
    // Is the user disabling forwarding?
    $disable = getvar('disable');
    
    $success = true;
    $msg = NULL;

    if (!$disable) {
        // Did we get an address?
        if (!$newAddress && !$removeAddress) {
            $msg = handle_error(7);
            $success = false;
        }
        elseif (!$removeAddress) {
            // Is the forwarding address valid?
            $emailValid = email_valid($newAddress);
            if (!$emailValid) {
                $msg = handle_error(5);
                $success = false;
            }
            $mailhost = explode('@', $newAddress);
            $mailhost = $mailhost[1];
            $hostValid = host_valid($mailhost);
            if (!$hostValid) {
                $msg = handle_error(5);
                $success = false;
            }
        }
    }

    if ($success) {
        commit_forward($newAddress, $disable, $removeAddress);
    }
    else {
        $response = array();
        $response['message'] = '<div class="webtools-container ui-state-error ui-corner-all">'.$msg.'</div>';
        $response['new_content'] = '';
        $response['update_content'] = 'no';
        send_response($response);
    }
}   

function commit_forward($newAddress, $disable, $removeAddress) {

    $rcmail = rcmail::get_instance();

    $USERNAME = $rcmail->config->get('USERNAME');
    $HOME = $rcmail->config->get('HOME');
    $RUNAS_CMD = $rcmail->config->get('RUNAS_CMD');
    $MF = $rcmail->config->get('MF');

    // Get some needed POST variables
    $forwardType = getvar('forwardType');
    $vacation = getvar('vacation');
    $newFormatVacation_S = getvar('newFormatVacation_S');    

    // Initialize other variables
    $insert = '';
    $createForward = true;
    $allAddr_Arr = array();
    $type = '';
    $oldemail = '';
    $msg = '';
    $status = 0;
    $dupe = 0;
    $user_cmd = '';
    $success = true;
    $hasVacation = false;
    $newVacationFileExist = false;

    // This is the response that will be sent back to the client
    $response = array('message' => '', 'new_content' => '', 'update_content' => 'yes');

    // Set insert to display some rutgers specific comments
    $insert = "################################################\n";
    $insert .= "# THIS FILE WAS CREATED BY A WEBTOOL.          #\n";
    $insert .= "# DO NOT EDIT THIS FILE BY HAND.               #\n";
    $insert .= "################################################\n\n";

    // Does the user have an existng vacation rule in an old-format file?
    if ($forwardType && $vacation && !$newFormatVacation_S) {
        $insert .= getVacationRule();
        $hasVacation = true;
    }

    // Are we adding on top of new formatted forward file?
    if ($newFormatVacation_S) {
        $hold = unserialize(base64_decode($newFormatVacation_S));
        $insert .= getVacationRule(NULL, $hold);
        $hasVacation = true;
    }

    if ($disable) {
        if (!$hasVacation) {
            $createForward = false;
        }
    }
    else {
        // Is the user keeping a local copy?
        $localCopy = getvar('localCopy');
        $localCopyPrior = getvar('localCopyPrior');

        // Old-format forward file addresses
        $oldFormatAddr_Arr = getvar('oldFormatAddr_Arr');

        // New-format forward file vars
        $newFormatAddr_Arr = getvar('newFormatAddr_Arr');
        $newVacationFileExist = getvar('newVacationFileExist');
        
        // Get any existing addresses from an old-format forward file
        if ($forwardType && $oldFormatAddr_Arr && is_array($oldFormatAddr_Arr)) {
            $allAddr_Arr = array_merge(array_keys($oldFormatAddr_Arr), $allAddr_Arr);
        }

        // Get any existing addresses from a new-format forward file
        if ($newFormatAddr_Arr && is_array($newFormatAddr_Arr)) {
            $allAddr_Arr = array_merge(array_keys($newFormatAddr_Arr), $allAddr_Arr);
        }

        if (!$removeAddress) {

            // Set server information
            $SERVER_NAME = $_SERVER['SERVER_NAME'];
            $IP = gethostbyname($SERVER_NAME);
            $server = gethostbyaddr($IP);

            // Prevent people from forwarding to themselves
            $oldemail = ("$USERNAME@$server");
            for ($index = 0; $index < count($allAddr_Arr); $index++) {
                if (strtolower($allAddr_Arr[$index]) == strtolower($oldemail)) {
                    $success = false;
                    $createForward = false;
                    $dupe = 1;
                    $msg .= '<strong>The forwarding address is the same as your current email address!</strong><br />';
                }
            }
            
            // Prevent people from forwarding to duplicate emails
            for ($index = 0; $index < count($allAddr_Arr); $index++) {
                if (strtolower($allAddr_Arr[$index]) == strtolower($newAddress)) {
                    $success = false;
                    $createForward = false;
                    $dupe = 1;
                    $msg .= '<strong>You are already forwarding your email to that specified address!</strong><br />';
                }
            }

            // Finally add the new address to the all addresses array
            array_push($allAddr_Arr, $newAddress);
        }
        else {
            
            $found = 0;

            // Remove the desired address
            for ($index = 0; $index < count($allAddr_Arr); $index++) {
                if (strtolower($allAddr_Arr[$index]) == strtolower($removeAddress)) {
                    array_splice($allAddr_Arr, $index, 1);
                    $found = 1;
                    break;
                }
            }
                      
            if (!$found) {
                $success = false;
                $createForward = false;
                $msg .= 'No addresses have been removed!<br />';
            }
            else {
                if (empty($allAddr_Arr)) {
                    $disable = true; 
                    if (!$hasVacation) {
                        $createForward = false;
                    }
                }
                $msg .= 'The address has been successfully removed!<br /><br />';
            }

        }

        if (!$disable) {
            // Are we leaving a local copy?
            if ((!$removeAddress && $localCopy === 'on') || ($removeAddress && $localCopyPrior === 'on')) {
                $addresses = implode(' ', $allAddr_Arr);
                $insert .= "cc \"!$addresses\"\n";
            }
            else {
                $addresses = implode(' ', $allAddr_Arr);
                $insert .= "to \"!$addresses\"\n";
            }
        }

    }

    // Should we write the new forward file?
    if ($createForward) {
        $user_cmd = "$MF Maildir/mailfilter-forward";
        $exec_target = "$RUNAS_CMD $USERNAME $user_cmd";
        unipipe_exec($exec_target, $insert, $status);
        if ($status != 0) {
            $msg .= 'Could not create your new forward/vacation file.<br />';
            $success = false;
        }
        elseif (!$disable) {
            $msg .= 'Your new forward file was created.<br />';
        }
        else {
            $msg .= 'Mail forwarding has been successfully disabled.<br />';
        }
    }
    elseif ($success && $disable) {
        // No vacation rule, so just remove the file
        $user_cmd = "find $HOME -maxdepth 2 -path $HOME/Maildir/mailfilter-forward -exec rm -f '{}' \;";
        $exec_target = "$RUNAS_CMD $USERNAME $user_cmd";
        exec($exec_target, $result, $status);
        if ($status != 0) {
            $msg .= 'Tried removing your forward file, but failed.<br />';
            $success = FALSE;
        }
        else {
            $msg .= 'Removed your forward file.<br />';
        }
    }

    // Move the old-format vacation file if it exists
    if ($success && $newVacationFileExist) {
        $user_cmd = "find $HOME -maxdepth 2 -path $HOME/Maildir/mailfilter-vacation -exec mv '{}' $HOME/Maildir/mailfilter-vacation-old \;";
        $exec_target = "$RUNAS_CMD $USERNAME $user_cmd";
        exec($exec_target, $result, $status);
        if ($status != 0) {
            $msg .= 'Could not move your old mailfilter-vacation file to mailfilter-vacation-old<br />';
            $success = false;
        }
        else {
            $msg .= 'Moved your old mailfilter-vacation file to mailfilter-vacation-old<br />';
        }
    }
        
    // Move the old-format forward file if it exists
    if ($success && $forwardType) {
        switch($forwardType) {
            case 'f':
                $user_cmd = "find $HOME -maxdepth 1 -name .forward -exec mv '{}' $HOME/.forward-old-webtool \;";
                $type = '.forward';
                break;
            case 'q':
                    $user_cmd = "find $HOME -maxdepth 1 -name .qmail -exec mv '{}' $HOME/.qmail-old-webtool \;";
                    $type = '.qmail';
                    break;
        }

        $exec_target = "$RUNAS_CMD $USERNAME $user_cmd";
        exec($exec_target, $result, $status);

        if ($status != 0) {
            $msg .= "Could not move your old $type file to $type-old-webtool.<br />";
            $success = false;
        }
        else {
            $msg .= "Your old $type file has been moved to $type-old-webtool.<br />";
        }
    }

    if ($success) {
        $response['message'] = '<div class="ui-state-highlight ui-corner-all" style="padding:10px; font-size:large;"><strong>Thank you '.$USERNAME.',<br /><br />'.$msg.'</strong>';
        
        if (!$disable) {
            // Update the address checkboxes
            $response['new_content'] .= '<strong>Leave these addresses checked to continue forwarding to them:</strong><br /><br />';
            foreach ($allAddr_Arr as $address) {
                $response['new_content'] .=
                    '<input type="checkbox" class="webtools-checkbox address-checkbox" id="checkbox_'.$address.'" name="newFormatAddr_Arr['.$address.']" "checked">
                    <label for="checkbox_'.$address.'">'.$address.'</label>
                    <a href="#" onclick="forward_post(false, \''.$address.'\'); return false;" title="Remove this address">
                        <span id="remove-address-icon" class="ui-icon ui-icon-trash" style="display:inline-block;"></span>
                    </a>
                    <br />';
            }
            if (!$removeAddress) {
                $response['new_content'] .= '<input type="hidden" id="localCopyPrior" name="localCopyPrior" value="'.$localCopy.'" />';
            }
            else {
                $response['new_content'] .= '<input type="hidden" id="localCopyPrior" name="localCopyPrior" value="'.$localCopyPrior.'" />';
            }

            $response['new_content'] .= '<br /><a href="#" onclick="forward_post(true); return false;">Remove all</a><br /><br />';
        }
    }
    else {
        $response['message'] = '<div class="webtools-container ui-state-error ui-corner-all"><strong>Sorry '.$USERNAME.',</strong><br /><br />'.$msg;
        $response['update_content'] = 'no';
    }
    $response['message'] .= '</div>';

    send_response($response);
}

function send_response($response) {
    $rcmail = rcmail::get_instance();
    $rcmail->output->command('plugin.forward-post-callback', $response);    
}

?>
